package me.teridax.jcash.banking;

import me.teridax.jcash.decode.StringUtils;

import java.util.Objects;

@SuppressWarnings("unused")
public abstract class Account {

    private final int iban;
    private final int pin;
    private double balance;

    public Account(int iban, int pin, double balance) {
        this.iban = iban;
        this.pin = pin;
        this.balance = balance;
    }

    public static Account fromColumns(String[] columns) {
        Objects.requireNonNull(columns);

        var iban = StringUtils.decodeUniqueIdentificationNumber(columns[0]);
        var pin = StringUtils.decodeUniqueIdentificationNumber(columns[1]);
        var balance = StringUtils.decodeCurrency(columns[2]);
        var type = StringUtils.decodeName(columns[3]);

        try {
            if (type.equals("Sparkonto")) {
                var interest = StringUtils.decodePercent(columns[4]);
                return new SavingsAccount(iban, pin, balance, interest);
            } else if (type.equals("Girokonto")) {
                var overdraft = StringUtils.decodeCurrency(columns[5]);
                return new Girokonto(iban, pin, balance, overdraft);
            } else {
                throw new IllegalArgumentException("Invalid account type: " + type);
            }
        } catch (IllegalArgumentException | NullPointerException e) {
            throw new IllegalArgumentException("Account format: ", e);
        }
    }

    public int getIban() {
        return iban;
    }

    public int getPin() {
        return pin;
    }

    public double getBalance() {
        return balance;
    }

    @Override
    public int hashCode() {
        return Objects.hash(iban);
    }

    @Override
    public boolean equals(Object obj) {
        if (obj instanceof Account)
            return iban == ((Account)obj).iban;

        return false;
    }

    public String getDescription() {
        return String.format("%s (%s)", iban, getClass().getSimpleName());
    }

    public void deposit(double amount) {
        this.balance += amount;
    }

    @Override
    public String toString() {
        return String.format("@Account [iban=%d, pin=%d, balance=%.2f]", iban, pin, balance);
    }

    public void takeoff(double amount) {
        this.balance = Math.max(0, this.balance - amount);
    }
}
